// editbar.cpp : implementation file
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1998 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"
//#include "docktool.h"
#include "editbar.h"
#include "resource.h"
#include "gariki.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#define COMBOBOX_INDEX 5
#define COMBOBOX_WIDTH 120
#define COMBOBOX_HEIGHT 150
#define COMBOBOX_BITMAP 4

static UINT BASED_CODE EditButtons[] =
{
	// same order as in the bitmap 'toolbar.bmp'
	ID_BACK,
	ID_NEXT,
	ID_COPY,
		ID_SEPARATOR,
	ID_FIND,
		ID_SEPARATOR,
		ID_SEPARATOR,
	ID_TOP_MOST,
	ID_SETUP,
	ID_HIDE,
		ID_SEPARATOR,
	ID_ABOUT,
};

/////////////////////////////////////////////////////////////////////////////
// CEditBar

CEditBar::CEditBar()
{
	m_bColor = TRUE;
}

CEditBar::~CEditBar()
{
}

/////////////////////////////////////////////////////////////////////////////
// CEditBar message handlers
BEGIN_MESSAGE_MAP(CEditBar, CToolBar)
	//{{AFX_MSG_MAP(CEditBar)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CEditBar::Init(CWnd* pParentWnd, BOOL bColor, BOOL bToolTips)
{
	m_bColor = bColor;
	m_bVertical = FALSE;

	if (!CreateEx(pParentWnd, TBSTYLE_FLAT , WS_CHILD | WS_VISIBLE 
															| CBRS_TOP | CBRS_TOOLTIPS | CBRS_SIZE_DYNAMIC))
	{
		return FALSE;
	}
/*
	if (!SetColor(m_bColor))
		return FALSE;
*/
	CBitmap ToolBitmap;
	ToolBitmap.LoadBitmap(IDR_TOOLBAR1);
	SetBtnSkin(ToolBitmap,9);

	if(!SetButtons(EditButtons, sizeof(EditButtons)/sizeof(UINT)))
		return FALSE;
	

	CRect rect(-COMBOBOX_WIDTH, -COMBOBOX_HEIGHT, 0, 0);
	// The ID of the ComboBox is important for two reasons.  One, so you
	// can receive notifications from the control.  And also for ToolTips.
	// During HitTesting if the ToolBar sees that the mouse is one a child
	// control, the toolbar will lookup the controls ID and search for a
	// string in the string table with the same ID to use for ToolTips
	// and StatusBar info.
	if (!m_SearchBox.Create(WS_CHILD | CBS_DROPDOWN |
		CBS_AUTOHSCROLL | WS_VSCROLL | CBS_HASSTRINGS, rect, this,
		IDC_EDITBAR_SEARCHBOX))
	{
		return FALSE;
	}

	HFONT hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
	if (hFont == NULL)
		hFont = (HFONT)GetStockObject(ANSI_VAR_FONT);

	m_SearchBox.SendMessage(WM_SETFONT, (WPARAM)hFont);

	if(!SetHorizontal())
		return FALSE;

	SetWindowText("    V3.0");
	return TRUE;
}

BOOL CEditBar::SetHorizontal()
{
	m_bVertical = FALSE;

	SetBarStyle(GetBarStyle() | CBRS_ALIGN_TOP);

	SetButtonInfo(COMBOBOX_INDEX, IDC_EDITBAR_SEARCHBOX, TBBS_SEPARATOR, COMBOBOX_WIDTH);

	if (m_SearchBox.m_hWnd != NULL)
	{
		CRect rect,wndrect;
		GetItemRect(COMBOBOX_INDEX, rect);
		rect.bottom = (rect.bottom - 22) / 2 + 1;
		m_SearchBox.SetWindowPos(NULL, rect.left, rect.top+rect.bottom, 0, 0, SWP_NOZORDER|SWP_NOACTIVATE|SWP_NOSIZE|SWP_NOCOPYBITS);
		m_SearchBox.ShowWindow(SW_SHOW);
	}

	return TRUE;
}

BOOL CEditBar::SetVertical()
{
	m_bVertical = TRUE;

	SetButtonInfo(COMBOBOX_INDEX, ID_EDIT_FIND, TBBS_BUTTON, COMBOBOX_BITMAP);

	if (m_SearchBox.m_hWnd != NULL)
		m_SearchBox.ShowWindow(SW_HIDE);

	return TRUE;
}

BOOL CEditBar::SetColor(BOOL bColor)
{


	return TRUE;
}

CSize CEditBar::CalcDynamicLayout(int nLength, DWORD dwMode)
{
	// if we're committing set the buttons appropriately
	if (dwMode & LM_COMMIT)
	{
		if (dwMode & LM_VERTDOCK)
		{
			if (!m_bVertical)
				SetVertical();
		}
		else
		{
			if (m_bVertical)
				SetHorizontal();
		}

		return CToolBar::CalcDynamicLayout(nLength, dwMode);
	}
	else
	{
		BOOL bOld = m_bVertical;
		BOOL bSwitch = (dwMode & LM_HORZ) ? bOld : !bOld;

		if (bSwitch)
		{
			if (bOld)
				SetHorizontal();
			else
				SetVertical();
		}


		CSize sizeResult = CToolBar::CalcDynamicLayout(nLength, dwMode);

		if (bSwitch)
		{
			if (bOld)
				SetHorizontal();
			else
				SetVertical();
		}

		return sizeResult;
	}
}


//CSize CEditBar::CalcFixedLayout(BOOL bStretch, BOOL bHorz)
//{
//      if (!bHorz && (m_dwStyle & CBRS_SIZE_DYNAMIC) && !(m_dwStyle & CBRS_FLOATING))
//  {
//      CSize size;
//
//      size = CControlBar::CalcFixedLayout(bStretch, bHorz);
//
//      CRect rect;
//      rect.SetRectEmpty();
//
//      CalcInsideRect(rect, bHorz);
//
//      size.cx = m_sizeVert.cx + (m_rectInsideVert.Width() - rect.Width());
//      size.cy = m_sizeVert.cy + (m_rectInsideVert.Height() - rect.Height());
//      return size;
//  }
//  else
//  {
//      if (m_bVertical)
//          SetHorizontal();
//
//      return CToolBar::CalcFixedLayout(bStretch, bHorz);
//  }
//}

//          
//  . ToolBitmap    .
//  -  , iNumBtns -    
BOOL CEditBar::SetBtnSkin(CBitmap &ToolBitmap, int iNumBtns)
{
	int wWidth, wHeight;

	CBitmap ActiveBmp, NormalBmp;
	BITMAP MainBmp;

	HIMAGELIST himlTemp;

	CImageList ActiveBtnList;
	CImageList DisableBtnList;

	ToolBitmap.GetBitmap(&MainBmp);
	//  
	wWidth = MainBmp.bmWidth / iNumBtns;
	wHeight = MainBmp.bmHeight / 2;
	//  
	if ((wWidth<10) || (wWidth>100) || (wHeight<10) || (wHeight>100))
			return FALSE;
	//   
	CSize sizeImage(wWidth, wHeight);
	CSize sizeButton(wWidth + 7, wHeight + 7);
	SetSizes(sizeButton, sizeImage);

	CDC MemDcDest, MemDcSrc;
	CDC *WinDC = GetDC();
	
	MemDcSrc.CreateCompatibleDC(WinDC);
	MemDcDest.CreateCompatibleDC(WinDC);

	MemDcSrc.SelectObject(&ToolBitmap);

	//  
	NormalBmp.CreateCompatibleBitmap(&MemDcSrc,wWidth * iNumBtns,wHeight);
	CBitmap *oldBmp = MemDcDest.SelectObject(&NormalBmp);
	MemDcDest.BitBlt(0,0,wWidth * iNumBtns,wHeight,&MemDcSrc,0,0,SRCCOPY);
	MemDcDest.SelectObject(oldBmp);
	
	DisableBtnList.Create(wWidth,wHeight,ILC_COLORDDB|ILC_MASK,iNumBtns,3);
	DisableBtnList.Add(&NormalBmp,RGB(255,0,255));
	himlTemp = (HIMAGELIST) SendMessage(TB_SETIMAGELIST, 0, (LPARAM)DisableBtnList.m_hImageList);
	::ImageList_Destroy(himlTemp);
	
	//  
	ActiveBmp.CreateCompatibleBitmap(&MemDcSrc,wWidth * iNumBtns,wHeight);
	oldBmp = MemDcDest.SelectObject(&ActiveBmp);
	MemDcDest.BitBlt(0,0,wWidth * iNumBtns,wHeight,&MemDcSrc,0,wHeight,SRCCOPY);
	MemDcDest.SelectObject(oldBmp);

	ActiveBtnList.Create(wWidth,wHeight,ILC_COLORDDB|ILC_MASK ,iNumBtns,3);
	ActiveBtnList.Add(&ActiveBmp,RGB(255,0,255));
	himlTemp = (HIMAGELIST) 	SendMessage(TB_SETHOTIMAGELIST, 0, (LPARAM)ActiveBtnList.m_hImageList);
	::ImageList_Destroy(himlTemp);	

	ActiveBtnList.Detach();
	DisableBtnList.Detach();

	return TRUE;
}

void CEditBar::OnPaint() 
{
	if (m_bDelayedButtonLayout)
		Layout();

	CPaintDC dc(this); // device context for paintin
	DefWindowProc(WM_PAINT,WPARAM(dc.m_hDC),0);

	// Separators hiding
	HideSeparators(&dc);
	// Separators hiding
}

//   
void CEditBar::HideSeparators(CDC *pDC)
{
	if(!::IsWindow(GetSafeHwnd()))
		return;

	CBrush* pFillBrush = GetBackgroundBrush();
	if(!pFillBrush)
		return;

	CRect ClipBox(0,0,0,0);
	pDC->GetClipBox(ClipBox);
	if(ClipBox.IsRectEmpty())
		return;

	for (HWND hWndChild = ::GetWindow(GetSafeHwnd(), GW_CHILD); hWndChild != NULL; hWndChild = ::GetWindow(hWndChild, GW_HWNDNEXT))
	{
		CRect ChildWindowRect;
		::GetWindowRect(hWndChild,ChildWindowRect);
		ScreenToClient(ChildWindowRect);

		int nToolBarItemCount = GetCount();
		for(int nI=0;nI<nToolBarItemCount;nI++)
		{
			UINT nID = 0;
			UINT nStyle = 0;
			int iImage = 0;

			GetButtonInfo(nI,nID,nStyle,iImage);

			if(nStyle != TBBS_SEPARATOR)
				continue;

			CRect ItemRect;
			GetItemRect(nI,ItemRect);

			if(!CRect().IntersectRect(ChildWindowRect,ItemRect))
				continue;

			CRect FillRect;
			if(!FillRect.IntersectRect(ItemRect,ClipBox))
				continue;

			pDC->FillRect(FillRect,pFillBrush);
		}
	}
}
//    CBrush    
//   
CBrush* CEditBar::GetBackgroundBrush()
{
	if(!::IsWindow(GetSafeHwnd()))
		return NULL;

	TCHAR szClassName[1024] = {NULL};

	WNDCLASSEX WndClassEx;
	ZeroMemory(&WndClassEx,sizeof(WndClassEx));
	WndClassEx.cbSize = sizeof(WndClassEx);

	if(::GetClassName(GetSafeHwnd(), 
					  szClassName, 
					  sizeof(szClassName)/sizeof(TCHAR)) &&
	   ::GetClassInfoEx(::AfxGetApp()->m_hInstance,
						szClassName,
						&WndClassEx))
		return CBrush::FromHandle(WndClassEx.hbrBackground);

	return NULL;
}
//       FilePath.  
//          
//   .        FilePath
//  m_SkinFilePath
void CEditBar::SetBtnSkinFile(CString FilePath, int iNumBtns)
{
	HBITMAP hBmp = (HBITMAP) LoadImage(AfxGetInstanceHandle(),(LPCSTR)FilePath,IMAGE_BITMAP,
														0,0,LR_LOADFROMFILE);
	if (hBmp != NULL)
	{	
			CBitmap ToolBmp;
			ToolBmp.Attach(hBmp);
			
			if (SetBtnSkin(ToolBmp,iNumBtns))
			{
				SetHorizontal();
				((CFrameWnd*)GetParent())->RecalcLayout(TRUE);
				//     
				m_SkinFilePath = FilePath;
				
				return;
			};
	}
	
	CString ErrStr;
	ErrStr.Format("    %s   ",FilePath);
	AfxMessageBox(ErrStr);
}
//   .       
//nIDResource. m_SkinFilePath   NULL
void CEditBar::SetBtnSkinRes(UINT nIDResource,int iNumBtns)
{
	CBitmap ToolBmp;
	ToolBmp.LoadBitmap(nIDResource);

	m_SkinFilePath.Empty();
	
	SetBtnSkin(ToolBmp,iNumBtns);
	SetHorizontal();
	((CFrameWnd*)GetParent())->RecalcLayout(TRUE);
}
// m_SkinFilePath
CString CEditBar::GetBtnSkinFilePath()
{
	return m_SkinFilePath;
}
